﻿<!DOCTYPE html>
<!--
###############################################################################
# The code below is released to public domain, you can do whatever you want
# with it and there is no need to refer to me as the original author.
#
# However, if you find it useful, I will be happy if you let me know :-)!
# Any comments to the code are also welcome. Especially if you find a bug
# or something which can be improved, I will be very happy to hear about it.
# You can contact me at papacut@kosmishmish.com.
#
# The code and the whole package is provided as it is, you acknowledge that it 
# can contain bugs. The author is neither responsible for any damage or loss,
# whether direct, consequential, incidental, or special which user suffers
# arising from any defect, error or fault of the application or its failure to
# perform, even if author has been advised of the possibility of such defect,
# error, fault, or failure. In any event, author's total aggregate liability
# to you for all damages of every kind and type (regardless of whether based in
# contract or tort) shall not exceed $0.00. In the case that applicable law may
# not allow the limitation or exclusion of liability or incidental or
# consequential damages, the above limitation or exclusion may not apply to
# you, although author's liability will be limited to the fullest extent
# permitted by applicable law. 
###############################################################################
-->
<html>

<style>
table, th, td {
  border: 1px solid black;
}
</style>

<head>
  <meta content="text/html; charset=UTF-8" http-equiv="content-type">
  <title>Web access counter</title>
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
</head>

<body>
    <h1>Counter statistics</h1>

    <?php

      require_once('setup_login.php');
      require_once('debug_support.php');

      $total = 0;
 
      $dbc = mysqli_connect('localhost', $C_USERNAME, $C_PASSWORD, $C_DATABASE);
      if (!$dbc) {
        echo (mysqli_connect_error()."<br>");
      } else {
        debug_echo("Connect succeeded.<br>");
      }

      #start transaction to ensure that the read data are consistent
      $result = mysqli_query($dbc, "START TRANSACTION READ ONLY");
      report_sql_error(!$result, "START TRANSACTION READ ONLY", "passed", "error");     
    ?>

    <!-- Page view counters -->

    <h2>Page view counters</h2>

    <table>
      <tr>
        <th>Page</th>
        <th>Counter</th>
      </tr>

      <?php
        $query = "SELECT * FROM web_counter_table";
        $result = mysqli_query($dbc, $query);
        report_sql_error(!$result, $query, "passed", "error");

        $counter_last = 0;
        $num_rows = mysqli_num_rows($result);
        if ((!$result)||($num_rows==0)) {
            $counter_last = 0;
            debug_echo("No rows found.<br>");
        } else {
            while ($row = mysqli_fetch_row($result)) {
                printf("<tr><td>%s</td><td>%05d</td></tr>\n", $row[0], $row[1]);
                $total = $total + $row[1];
            }
        }
        printf("<tr><td><b>Total page views</b></td><td><b>%05d</b></td></tr>", $total);
      ?>
    </table>

    <!-- WEB PAGE TRACES -->

    <h2>Web page traces</h2>

    <ul>
      <li><span style="color:red;font-weight:bold">Red hash</span>: a new hash value.</li>
      <li><span style="color:green">Green hash</span>: a new unique user with the same hash, detected using the timeout.</li>   
      <li>No hash (empty): next access from the same user as on the previous line.</li>
      <li><span style="color:red">Red After</span>: time which elapsed since the last moment we detected
       access from an IP with the same hash, bigger than timeout. A new unique user trace begins here.</li>
      <li><span style="color:green;font-weight:bold">Green After</span>: time between two accesses of the same user.</li>   
    </ul>

    <table>
      <tr>
        <th>User<br>hash</th>
        <th>Displayed page</th>
        <th>When</th>
        <th>After<br> [min]</th>
      </tr>

    <?php
      $query = "SELECT * FROM traces ORDER BY hash, request_time";
      $result = mysqli_query($dbc, $query);
      report_sql_error(!$result, $query, "passed", "error");

      $last_hash = -1;
      $last_time = 0;
      $unique_users = 0;

      $num_rows = mysqli_num_rows($result);
      if ((!$result)||($num_rows==0)) {
        debug_echo("No rows found.<br>");
      } else {
        while ($row = mysqli_fetch_row($result)) {
          $hash = $row[0];
          $request_time = $row[1];
          $url = $row[2];

          #is it a new hash or the timeout elapsed since the last visit?
          $new_user = false;

          printf("<tr>");

          if ($hash != $last_hash) {
            #a new hash - a new unique user
            printf("<td style=\"color:red;font-weight:bold\">%03d</td>",$hash);
            $new_user = true;
            $last_time = $request_time;
          } else if (($request_time - $last_time) > $C_VISITOR_TIMEOUT) {
            #timeout elapsed - a new unique user
            printf("<td style=\"color:green\">%03d</td>",$hash);
            $new_user = true;
          } else {           
            #the same as on the previous line, keep the cell in the table empty   
            printf("<td></td>");
          }

          $delta_minutes = (($request_time-$last_time)/60);
          $last_hash = $hash;
          $last_time = $request_time;

          printf("<td>%s</td>", $url);
          printf("<td>%s</td>", date("d.m.y, H:i:s",$request_time));

          if ($new_user) {
            printf("<td style=\"color:red\">%4.1f</td>", $delta_minutes);
            $unique_users = $unique_users + 1;
          } else {
            printf("<td style=\"color:green;font-weight:bold\">%4.1f</td>", $delta_minutes);
          }
        }
        printf("</tr>");
      }

      printf("<td></td><td><b>Unique users with %02d sec. timeout<br>during the last %02d days.</b></td><td><b>%04d</b></td><td></td>",
       $C_VISITOR_TIMEOUT,
       $C_DELETE_PERIOD/(24*60*60),
       $unique_users);
    ?>
  </table>

  <h2>Unique user counters</h2>

  <table style="border: 1px solid black">
    <tr>
      <th>Hash</th>
      <th>Time of the last request</th>
      <th>No. of unqiue<br> visitors</th>
    </tr>

    <?php

      $query = "SELECT * FROM user_hashes";
      $result = mysqli_query($dbc, $query);
      report_sql_error(!$result, $query, "passed", "error");      

      if ($debug) {
        print_r($result);
      };

      $total = 0;
      $hash_count = 0;
      $num_rows = mysqli_num_rows($result);
      if ((!$result)||($num_rows==0)) {
        debug_echo("NOT FOUND<br>");
      } else {
        while ($row = mysqli_fetch_row($result)) {
          printf("<tr><td>%3d</td><td>%s</td><td>%05d</td></tr>\n", $row[0], date("d.m.y H:i:s",$row[1]), $row[2]);
          $total = $total + $row[2];
          $hash_count = $hash_count + 1;
        }
      }
      printf("<tr><td></td><td><b>Unique visitors</b></td><td><b>%05d</b></td></tr>", $total);
      printf("<tr><td></td><td><b>Unique hashes</b></td><td><b>%05d</b></td></tr>", $hash_count);
  ?> 
</table>    

<?php
  $result = mysqli_query($dbc, "COMMIT");
  report_sql_error(!$result, "COMMIT", "passed", "error");

  mysqli_close($dbc);
?>

<footer>
  <hr>
  Copyright &copy; 2015-2021 PaPa&#268;ut, <a href="mailto:papacut@kosmishmish.com">papacut@kosmishmish.com</a>, last modified on 29.12.2021.
</footer>

</body>